/***************************************************************************
                          brightnesscontrastdialog.cpp  -  description
                             -------------------
    begin                : Mon Jan 21 2002
    copyright            : (C) 2002 by Michael Herder
    email                : crapsite@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/*****************************************************************************
 * The algorithm used in this class is taken from the Gimp 1.2.0             *
 * Original filename: lut_func.c                                             *
 * Original copyright message:                                               *
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis                       *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify      *
 * it under the terms of the GNU General Public License as published by      *
 * the Free Software Foundation; either version 2 of the License, or         *
 * (at your option) any later version.                                       *
 *                                                                           *
 * This program is distributed in the hope that it will be useful,           *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 * GNU General Public License for more details.                              *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program; if not, write to the Free Software               *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.*
 *****************************************************************************/

#include "brightnesscontrastdialog.h"
#include "quiteinsane/qxmlconfig.h"
#include "quiteinsane/sliderspin.h"

#include <math.h>

#include <qapplication.h>
#include <qhbox.h>
#include <qimage.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qnamespace.h>
#include <qpixmap.h>
#include <qpushbutton.h>
#include <qvbox.h>

BrightnessContrastDialog::BrightnessContrastDialog(int preview_size,QImage* image,QWidget* parent)
                         :ImageFilterDialog(preview_size,image,parent)
{
  mMasterArray.resize(256);
  mContrastArray.resize(256);
  mBrightnessArray.resize(256);
  for(int i=0;i<256;i++)
  {
    mMasterArray[i] = i;
    mContrastArray[i] = i;
    mBrightnessArray[i] = i;
  }
  initControls();
}
BrightnessContrastDialog::~BrightnessContrastDialog()
{
}
/** No descriptions */
void BrightnessContrastDialog::initControls()
{
  int brightness = xmlConfig->intValue("FILTER_BRIGHTNESSCONTRAST_BRIGHTNESS",0);
  int contrast = xmlConfig->intValue("FILTER_BRIGHTNESSCONTRAST_CONTRAST",0);
  bool c_up = xmlConfig->boolValue("FILTER_BRIGHTNESSCONTRAST_CONTINOUS_UPDATE",false);

  QVBox* vb = controlsVBox();
  if(!vb)
    return;
  setTitle(tr("Brightness/Contrast"));
  setCaption(tr("Brightness/Contrast"));
  //brightness
  mpBrightnessSliderSpin = new SliderSpin(-127,127,brightness,tr("Brightness"),vb);
  mpBrightnessSliderSpin->setMinimumWidth(150);
  //contrast
  mpContrastSliderSpin = new SliderSpin(-127,127,contrast,tr("Contrast"),vb);
  mpContrastSliderSpin->setMinimumWidth(150);
  //reset button
  mpResetButton = new QPushButton(tr("Reset"),vb);

  QWidget* dummy = new QWidget(vb);
  vb->setStretchFactor(dummy,1);
  connect(mpResetButton,SIGNAL(clicked()),
          this,SLOT(slotReset()));
  connect(mpBrightnessSliderSpin,SIGNAL(signalValueChanged(int)),
          this,SLOT(slotBrightnessChanged(int)));
  connect(mpContrastSliderSpin,SIGNAL(signalValueChanged(int)),
          this,SLOT(slotContrastChanged(int)));
  //load values
  slotBrightnessChanged(brightness);
  slotContrastChanged(contrast);
  setContinousUpdate(c_up);
  setFixedSize(minimumSizeHint());
}
/** No descriptions */
bool BrightnessContrastDialog::apply(QImage* image,bool emit_progress)
{
  int x,y;
  QRgb rgb_val;
  int progress,progresscnt;
  progress = image->width() * image->height();
  progresscnt = 0;
  int old_p = 0;

  for (int i=0;i<256;i++)
    mMasterArray[i] = mContrastArray[mBrightnessArray[i]];

  if(image->isNull() || image->depth()<8) return false;
  if(image->numColors() > 0)
  {
    //has palette
    int num;
    for(num=0;num<image->numColors();num++)
    {
      rgb_val = image->color(num);
      image->setColor(num,qRgba(mMasterArray[qRed(rgb_val)],
                                mMasterArray[qGreen(rgb_val)],
                                mMasterArray[qBlue(rgb_val)],
                                qAlpha(rgb_val)));
    }
  }
  else
  {
    for (y=0; y < image->height(); y++)
    {
      for (x=0; x < image->width(); x++)
      {
        rgb_val = image->pixel(x,y);
        image->setPixel(x,y,qRgba(mMasterArray[qRed(rgb_val)],
                                  mMasterArray[qGreen(rgb_val)],
                                  mMasterArray[qBlue(rgb_val)],
                                  qAlpha(rgb_val)));
        if(emit_progress)
        {
          if(stopped())
            return false;
          ++progresscnt;
          int p = int(100.0*double(progresscnt)/double(progress));
          if((p % 5 == 0) && (p > old_p))
          {
            old_p = p;
            emit signalFilterProgress(p);
            qApp->processEvents();
          }
        }
      }
    }
  }
  return true;
}
/**  */
void BrightnessContrastDialog::slotContrastChanged(int contrast)
{
  int val;
  double p;
	int cnt;

	for (cnt=0;cnt < 256;cnt++)
  {
    if(contrast < 0)
    {
      val = (cnt > 127) ? (255 - cnt) : cnt;
      val = (int) (127.0 * pow ((double) (val ? val : 1) / 127.0,
                                (double) (127 + contrast) / 127.0));
      mContrastArray[cnt] = (cnt > 127) ? (255 - val) : val;
      if(mContrastArray[cnt] > 255) mContrastArray[cnt] = 255;
      if(mContrastArray[cnt] < 0) mContrastArray[cnt] = 0;
    }
    else
    {
      val = (cnt > 127) ? (255 - cnt) : cnt;
      p = (contrast == 127) ? 127 : 127.0 / (127 - contrast);
      val = (int) (127.0 * pow ((double) val / 127.0, p));
      mContrastArray[cnt] = (cnt > 127) ? (255 - val) : val;
      if(mContrastArray[cnt] > 255) mContrastArray[cnt] = 255;
      if(mContrastArray[cnt] < 0) mContrastArray[cnt] = 0;
    }
  }
  updatePreview();
}
/**  */
void BrightnessContrastDialog::slotBrightnessChanged(int brightness)
{
	int cnt;

	for (cnt=0;cnt< 256;cnt++)
  {
    if(brightness < 0)
      mBrightnessArray[cnt] = ((cnt * (255 + brightness)) / 255);
    else
      mBrightnessArray[cnt] = (cnt + ((255 - cnt) * brightness) / 255);
    if(mBrightnessArray[cnt] > 255) mBrightnessArray[cnt] = 255;
    if(mBrightnessArray[cnt] < 0) mBrightnessArray[cnt] = 0;
  }
  updatePreview();
}
/** No descriptions */
void BrightnessContrastDialog::saveConfig()
{
  xmlConfig->setIntValue("FILTER_BRIGHTNESSCONTRAST_BRIGHTNESS",
                         mpBrightnessSliderSpin->value());
  xmlConfig->setIntValue("FILTER_BRIGHTNESSCONTRAST_CONTRAST",
                         mpContrastSliderSpin->value());
  xmlConfig->setBoolValue("FILTER_BRIGHTNESSCONTRAST_CONTINOUS_UPDATE",
                          continousUpdate());
}
/** No descriptions */
void BrightnessContrastDialog::slotReset()
{
  mpBrightnessSliderSpin->setValue(0);
  mpContrastSliderSpin->setValue(0);
}
